
pub mod global;
pub mod helper;


use pico_args::Arguments;
use evalexpr::{
	context_map,

	ContextWithMutableVariables,
	HashMapContext,
	Value
};

use crate::flag;

pub fn build(args: &mut Arguments) -> HashMapContext {
	let mut output =
		if !args.contains(flag::EMPTY_CONTEXT) {
			context_map! {
				//	globals
				"c"		=>	global::LIGHT_SPEED,
				"e"		=>	global::EULER,
				"phi"	=>	global::GOLDEN_RATIO,
				"pi"	=>	global::PI,
				"√2"	=>	global::ROOT_TWO,

				//	math functions
				"cos"	=>	Function::new(|arg| helper::cosine(arg)),
				"fix"	=>	Function::new(|arg| helper::fix(arg)),
				"log"	=>	Function::new(|arg| helper::logarithm(arg)),
				"sin"	=>	Function::new(|arg| helper::sine(arg)),
				"sqrt"	=>	Function::new(|arg| helper::square_root(arg)),
				"tan"	=>	Function::new(|arg| helper::tangent(arg)),

				//	data science functions
				"avg"	=>	Function::new(|arg| helper::average(arg)),

				//	radix functions
				"bin"	=>	Function::new(|arg| helper::binary(arg)),
				"hex"	=>	Function::new(|arg| helper::hexadecimal(arg)),
				"oct"	=>	Function::new(|arg| helper::octal(arg)),

				//	character aliases
				"ϕ"		=>	global::GOLDEN_RATIO,
				"π"		=>	global::PI,
				"√"		=>	Function::new(|arg| helper::square_root(arg))
			}.unwrap()
		} else { HashMapContext::new() };

	while let Ok(value) = args.value_from_str::<&str, String>(flag::SET) {
		let split: Vec<&str> = value.split('=').collect();
		if split.len() == 2 {
			let key = split[0].to_owned();

			let value_str = split[1];
			let value =
				if let Ok(integer) = value_str.parse::<i64>() { Value::Int(integer) }
				else if let Ok(float) = value_str.parse::<f64>() { Value::Float(float) }
				else { Value::from(value_str) };

			output.set_value(key, value).ok(); }
		else { std::process::exit(1); }
	}

	output
}

